/* Copyright (c) 2023 ZAO "MCST". All rights reserved. */

#ifndef _CPUID_H_INCLUDED
#define _CPUID_H_INCLUDED

/* %ecx */
#define bit_SSE3	(1 << 0)
#define bit_PCLMUL	(1 << 1)
#define bit_LZCNT	(1 << 5)
#define bit_SSSE3	(1 << 9)
#define bit_FMA		(1 << 12)
#define bit_CMPXCHG16B	(1 << 13)
#define bit_SSE4_1	(1 << 19)
#define bit_SSE4_2	(1 << 20)
#define bit_MOVBE	(1 << 22)
#define bit_POPCNT	(1 << 23)
#define bit_AES		(1 << 25)
#define bit_XSAVE	(1 << 26)
#define bit_OSXSAVE	(1 << 27)
#define bit_AVX		(1 << 28)
#define bit_F16C	(1 << 29)
#define bit_RDRND	(1 << 30)

/* %edx */
#define bit_CMPXCHG8B	(1 << 8)
#define bit_CMOV	(1 << 15)
#define bit_MMX		(1 << 23)
#define bit_FXSAVE	(1 << 24)
#define bit_SSE		(1 << 25)
#define bit_SSE2	(1 << 26)

/* Extended Features (%eax == 0x80000001) */
/* %ecx */
#define bit_LAHF_LM	(1 << 0)
#define bit_ABM		(1 << 5)
#define bit_SSE4a	(1 << 6)
#define bit_PRFCHW	(1 << 8)
#define bit_XOP		(1 << 11)
#define bit_LWP		(1 << 15)
#define bit_FMA4	(1 << 16)
#define bit_TBM		(1 << 21)
#define bit_MWAITX	(1 << 29)

/* %edx */
#define bit_MMXEXT	(1 << 22)
#define bit_LM		(1 << 29)
#define bit_3DNOWP	(1 << 30)
#define bit_3DNOW	(1u << 31)

/* %ebx  */
#define bit_CLZERO	(1 << 0)
#define bit_RDPRU	(1 << 4)
#define bit_WBNOINVD	(1 << 9)

/* Extended Features Leaf (%eax == 7, %ecx == 0) */
/* %ebx */
#define bit_FSGSBASE	(1 << 0)
#define bit_SGX		(1 << 2)
#define bit_BMI		(1 << 3)
#define bit_HLE		(1 << 4)
#define bit_AVX2	(1 << 5)
#define bit_BMI2	(1 << 8)
#define bit_RTM		(1 << 11)
#define bit_MPX		(1 << 14)
#define bit_AVX512F	(1 << 16)
#define bit_AVX512DQ	(1 << 17)
#define bit_RDSEED	(1 << 18)
#define bit_ADX		(1 << 19)
#define bit_AVX512IFMA	(1 << 21)
#define bit_CLFLUSHOPT	(1 << 23)
#define bit_CLWB	(1 << 24)
#define bit_AVX512PF	(1 << 26)
#define bit_AVX512ER	(1 << 27)
#define bit_AVX512CD	(1 << 28)
#define bit_SHA		(1 << 29)
#define bit_AVX512BW	(1 << 30)
#define bit_AVX512VL	(1u << 31)

/* %ecx */
#define bit_PREFETCHWT1	(1 << 0)
#define bit_AVX512VBMI	(1 << 1)
#define bit_PKU		(1 << 3)
#define bit_OSPKE	(1 << 4)
#define bit_WAITPKG	(1 << 5)
#define bit_AVX512VBMI2	(1 << 6)
#define bit_SHSTK	(1 << 7)
#define bit_GFNI	(1 << 8)
#define bit_VAES	(1 << 9)
#define bit_VPCLMULQDQ	(1 << 10)
#define bit_AVX512VNNI	(1 << 11)
#define bit_AVX512BITALG	(1 << 12)
#define bit_AVX512VPOPCNTDQ	(1 << 14)
#define bit_RDPID	(1 << 22)
#define bit_KL		(1 << 23)
#define bit_CLDEMOTE	(1 << 25)
#define bit_MOVDIRI	(1 << 27)
#define bit_MOVDIR64B	(1 << 28)
#define bit_ENQCMD	(1 << 29)

/* %edx */
#define bit_AVX5124VNNIW	(1 << 2)
#define bit_AVX5124FMAPS	(1 << 3)
#define bit_UINTR	(1 << 5)
#define bit_AVX512VP2INTERSECT	(1 << 8)
#define bit_SERIALIZE	(1 << 14)
#define bit_TSXLDTRK	(1 << 16)
#define bit_PCONFIG	(1 << 18)
#define bit_IBT		(1 << 20)
#define bit_AMX_BF16	(1 << 22)
#define bit_AVX512FP16	(1 << 23)
#define bit_AMX_TILE	(1 << 24)
#define bit_AMX_INT8	(1 << 25)

/* Extended Features Sub-leaf (%eax == 7, %ecx == 1) */
/* %eax */
#define bit_SHA512	(1 << 0)
#define bit_SM3		(1 << 1)
#define bit_SM4		(1 << 2)
#define bit_RAOINT	(1 << 3)
#define bit_AVXVNNI	(1 << 4)
#define bit_AVX512BF16	(1 << 5)
#define bit_CMPCCXADD	(1 << 7)
#define bit_AMX_COMPLEX	(1 << 8)
#define bit_AMX_FP16	(1 << 21)
#define bit_HRESET	(1 << 22)
#define bit_AVXIFMA	(1 << 23)

/* %edx */
#define bit_AVXVNNIINT8	(1 << 4)
#define bit_AVXNECONVERT	(1 << 5)
#define bit_AVXVNNIINT16	(1 << 10)
#define bit_PREFETCHI	(1 << 14)
#define bit_USER_MSR	(1 << 15)
#define bit_AVX10	(1 << 19)
#define bit_APX_F	(1 << 21)

/* XFEATURE_ENABLED_MASK register bits (%eax == 0xd, %ecx == 0) */
#define bit_BNDREGS	(1 << 3)
#define bit_BNDCSR	(1 << 4)

/* Extended State Enumeration Sub-leaf (%eax == 0xd, %ecx == 1) */
#define bit_XSAVEOPT	(1 << 0)
#define bit_XSAVEC	(1 << 1)
#define bit_XSAVES	(1 << 3)

/* PT sub leaf (%eax == 0x14, %ecx == 0) */
/* %ebx */
#define bit_PTWRITE	(1 << 4)

/* Keylocker leaf (%eax == 0x19) */
/* %ebx */
#define bit_AESKLE	(1 << 0)
#define bit_WIDEKL	(1 << 2)

/* AVX10 sub leaf (%eax == 0x24) */
/* %ebx */
#define bit_AVX10_256	(1 << 17)
#define bit_AVX10_512	(1 << 18)

/* Signatures for different CPU implementations as returned in uses
   of cpuid with level 0.  */
#define signature_AMD_ebx	0x68747541
#define signature_AMD_ecx	0x444d4163
#define signature_AMD_edx	0x69746e65

#define signature_CENTAUR_ebx	0x746e6543
#define signature_CENTAUR_ecx	0x736c7561
#define signature_CENTAUR_edx	0x48727561

#define signature_CYRIX_ebx	0x69727943
#define signature_CYRIX_ecx	0x64616574
#define signature_CYRIX_edx	0x736e4978

#define signature_INTEL_ebx	0x756e6547
#define signature_INTEL_ecx	0x6c65746e
#define signature_INTEL_edx	0x49656e69

#define signature_TM1_ebx	0x6e617254
#define signature_TM1_ecx	0x55504361
#define signature_TM1_edx	0x74656d73

#define signature_TM2_ebx	0x756e6547
#define signature_TM2_ecx	0x3638784d
#define signature_TM2_edx	0x54656e69

#define signature_NSC_ebx	0x646f6547
#define signature_NSC_ecx	0x43534e20
#define signature_NSC_edx	0x79622065

#define signature_NEXGEN_ebx	0x4778654e
#define signature_NEXGEN_ecx	0x6e657669
#define signature_NEXGEN_edx	0x72446e65

#define signature_RISE_ebx	0x65736952
#define signature_RISE_ecx	0x65736952
#define signature_RISE_edx	0x65736952

#define signature_SIS_ebx	0x20536953
#define signature_SIS_ecx	0x20536953
#define signature_SIS_edx	0x20536953

#define signature_UMC_ebx	0x20434d55
#define signature_UMC_ecx	0x20434d55
#define signature_UMC_edx	0x20434d55

#define signature_VIA_ebx	0x20414956
#define signature_VIA_ecx	0x20414956
#define signature_VIA_edx	0x20414956

#define signature_VORTEX_ebx	0x74726f56
#define signature_VORTEX_ecx	0x436f5320
#define signature_VORTEX_edx	0x36387865

/* MCSTElbrus2K */
#define signature_MCSTE2K_ebx	0x5453434d
#define signature_MCSTE2K_ecx	0x4b327375
#define signature_MCSTE2K_edx	0x72626c45

/* intrinsic to check CPU vendor, family, features */
static __inline void
__cpuidex (int __cpuid_info[4], int __leaf, int __subleaf)
{
    if (__leaf == 1 && !__subleaf) {
        __cpuid_info[2] =
#ifdef __SSE3__
                          bit_SSE3 |
#endif
#ifdef __PCLMUL__
                          bit_PCLMUL |
#endif
#ifdef __LZCNT__
                          bit_LZCNT |
#endif
#ifdef __SSSE3__
                          bit_SSSE3 |
#endif
#ifdef __FMA__
                          bit_FMA |
#endif
#ifdef __SSE4_1__
                          bit_SSE4_1 |
#endif
#ifdef __SSE4_2__
                          bit_SSE4_2 |
#endif
#ifdef __POPCNT__
                          bit_POPCNT |
#endif
#ifdef __AES__
                          bit_AES |
#endif
                          bit_XSAVE |
                          bit_OSXSAVE |
#ifdef __AVX__
                          bit_AVX |
#endif
#ifdef __F16C__
                          bit_F16C |
#endif
#ifdef __RDRND__
                          bit_RDRND |
#endif
                          0;
        __cpuid_info[3] =
#ifdef __MMX__
                          bit_MMX |
#endif
#ifdef __SSE__
                          bit_SSE |
#endif
#ifdef __SSE2__
                          bit_SSE2 |
#endif
                          0;
        __cpuid_info[0] = __cpuid_info[1] = 0;
    }
    else if (__leaf == 7 && !__subleaf) {
        __cpuid_info[1] =
#ifdef __FSGSBASE__
                          bit_FSGSBASE |
#endif
#ifdef __SGX__
                          bit_SGX |
#endif
#ifdef __BMI__
                          bit_BMI |
#endif
#ifdef __AVX2__
                          bit_AVX2 |
#endif
#ifdef __BMI2__
                          bit_BMI2 |
#endif
#ifdef __AVX512F__
                          bit_AVX512F |
#endif
#ifdef __AVX512DQ__
                          bit_AVX512DQ |
#endif
#ifdef __RDSEED__
                          bit_RDSEED |
#endif
                          bit_ADX |
#ifdef __AVX512IFMA__
                          bit_AVX512IFMA |
#endif
#ifdef __CLFLUSHOPT__
                          bit_CLFLUSHOPT |
#endif
#ifdef __CLWB__
                          bit_CLWB |
#endif
#ifdef __AVX512PF__
                          bit_AVX512PF |
#endif
#ifdef __AVX512ER__
                          bit_AVX512ER |
#endif
#ifdef __AVX512CD__
                          bit_AVX512CD |
#endif
#ifdef __SHA__
                          bit_SHA |
#endif
#ifdef __AVX512BW__
                          bit_AVX512BW |
#endif
#ifdef __AVX512VL__
                          bit_AVX512VL |
#endif
                          0;
        __cpuid_info[0] = __cpuid_info[2] = __cpuid_info[3] = 0;
    }
    else if (__leaf == 0x80000000 && !__subleaf) {
        __cpuid_info[0] = 0x80000001;
        __cpuid_info[1] = __cpuid_info[2] = __cpuid_info[3] = 0;
    }
    else if (__leaf == 0x80000001 && !__subleaf) {
        __cpuid_info[1] =
#ifdef __CLZERO__
                          bit_CLZERO |
#endif
                          0;
        __cpuid_info[2] =
#ifdef __SSE4A__
                          bit_SSE4a |
#endif
                          bit_PRFCHW |
#ifdef __XOP__
                          bit_XOP |
#endif
#ifdef __FMA4__
                          bit_FMA4 |
#endif
#ifdef __TBM__
                          bit_TBM |
#endif
#ifdef __MWAITX__
                          bit_MWAITX |
#endif
                          0;
        __cpuid_info[3] =
#ifdef __3dNOW__
                          bit_3DNOWP |
                          bit_3DNOW |
#endif
                          0;
        __cpuid_info[0] = 0;
    }
    else if (__leaf == 0 && !__subleaf) {
        __cpuid_info[0] = 7;
        __cpuid_info[1] = signature_MCSTE2K_ebx;
        __cpuid_info[2] = signature_MCSTE2K_ecx;
        __cpuid_info[3] = signature_MCSTE2K_edx;
    } else {
        __cpuid_info[0] = __cpuid_info[1] = __cpuid_info[2] = __cpuid_info[3] = 0;
    }
}

/* Return cpuid data for requested cpuid leaf, as found in returned
   eax, ebx, ecx and edx registers.  The function checks if cpuid is
   supported and returns 1 for valid cpuid information or 0 for
   unsupported cpuid leaf.  All pointers are required to be non-null.  */

static __inline int
__get_cpuid_count (unsigned int __leaf, unsigned int __subleaf,
		   unsigned int *__eax, unsigned int *__ebx,
		   unsigned int *__ecx, unsigned int *__edx)
{
  int __info[4];

  __cpuidex (__info, __leaf & 0x80000000, 0);
  if (!__info[0] || __info[0] < __leaf)
      return 0;
  __cpuidex (__info, __leaf, __subleaf);
  *__eax = __info[0];
  *__ebx = __info[1];
  *__ecx = __info[2];
  *__edx = __info[3];
  return 1;
}

#define __get_cpuid(level, a, b, c, d) __get_cpuid_count (level, 0, a, b, c, d)

#define __cpuid_count(level, count, a, b, c, d) do {                    \
  int __info[4];                                                        \
                                                                        \
  __cpuidex (__info, level, count);                                     \
  a = __info[0];                                                        \
  b = __info[1];                                                        \
  c = __info[2];                                                        \
  d = __info[3];                                                        \
} while (0)

#define __cpuid(level, a, b, c, d) __cpuid_count (level, 0, a, b, c, d)

/* Return highest supported input value for cpuid instruction.  ext can
   be either 0x0 or 0x80000000 to return highest supported value for
   basic or extended cpuid information.  Function returns 0 if cpuid
   is not supported or whatever cpuid returns in eax register.  If sig
   pointer is non-null, then first four bytes of the signature
   (as found in ebx register) are returned in location pointed by sig.  */

static __inline unsigned int
__get_cpuid_max (unsigned int __ext, unsigned int *__sig)
{
  int __info[4];

  __cpuidex (__info, __ext, 0);
  if (__sig)
      *__sig = __info[1];
  return __info[0];
}

#endif /* _CPUID_H_INCLUDED */
