/* Copyright (c) 2018 ZAO "MCST". All rights reserved. */

#ifndef _EMMINTRIN_H_INCLUDED
#define _EMMINTRIN_H_INCLUDED

/* We need definitions from the SSE header files*/
#include <xmmintrin.h>

/* __SSE2__ */

/* Define the default attributes for the functions in this file */
#define __DEFAULT_FUNCS_ATTRS __attribute__((/*__gnu_inline__,*/ __always_inline__, __artificial__))

/* The Intel API is flexible enough that we must allow aliasing with other
   vector types, and their scalar components.  */
typedef long long __m128i __attribute__ ((__vector_size__ (16), __may_alias__));
typedef double __m128d __attribute__ ((__vector_size__ (16), __may_alias__));

/* Unaligned version of the same types.  */
typedef long long __m128i_u __attribute__ ((__vector_size__ (16), __may_alias__, __aligned__ (1)));
typedef double __m128d_u __attribute__ ((__vector_size__ (16), __may_alias__, __aligned__ (1)));

/* Create a selector for use with the SHUFPD instruction.  */
#define _MM_SHUFFLE2(fp1,fp0) \
 (((fp1) << 1) | (fp0))

/* Create a vector with element 0 as F and the rest zero.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_set_sd (double __F)
{
  return ((type_union_128) {d : { __F, 0.0 }}).__v2df;
}

/* Create a vector with both elements equal to F.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_set1_pd (double __F)
{
  return ((type_union_128) {d : { __F, __F }}).__v2df;
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_set_pd1 (double __F)
{
  return _mm_set1_pd (__F);
}

/* Create a vector with the lower value X and upper value W.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_set_pd (double __W, double __X)
{
  return ((type_union_128) {d : { __X, __W }}).__v2df;
}

/* Create a vector with the lower value W and upper value X.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_setr_pd (double __W, double __X)
{
  return ((type_union_128) {d : { __W, __X }}).__v2df;
}

/* Create an undefined vector.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_undefined_pd (void)
{
  __m128d __Y = __Y;
  return __Y;
}

/* Create a vector of zeros.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_setzero_pd (void)
{
  return __extension__ (__m128d){ 0.0, 0.0 };
}

/* Sets the low DPFP value of A from the low value of B.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_move_sd (__m128d __A, __m128d __B)
{
  return (__m128d) __builtin_ia32_movsd ((__v2df)__A, (__v2df)__B);
/*  return __extension__ (__m128d) __builtin_shuffle ((__v2df)__A, (__v2df)__B, (__v2di){2, 1});*/
}

/* Load two DPFP values from P.  The address must be 16-byte aligned.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_load_pd (double const *__P)
{
#if __iset__ <= 4
  return *(__m128d *) __builtin_assume_aligned (__P, 16);
#else /* __iset__ <= 4 */
  return (__m128d) __builtin_loadmas_128v (__P, 0, __LCC_CHAN_ANY);
#endif /* __iset__ <= 4 */
}

/* Load two DPFP values from P.  The address need not be 16-byte aligned.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_loadu_pd (double const *__P)
{
  return __builtin_ia32_loadupd (__P);
}

/* Create a vector with all two elements equal to *P.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_load1_pd (double const *__P)
{
  return _mm_set1_pd (*__P);
}

/* Create a vector with element 0 as *P and the rest zero.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_load_sd (double const *__P)
{
  return _mm_set_sd (*__P);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_load_pd1 (double const *__P)
{
  return _mm_load1_pd (__P);
}

/* Load two DPFP values in reverse order.  The address must be aligned.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_loadr_pd (double const *__P)
{
  __m128d __tmp = _mm_load_pd (__P);
  return __builtin_ia32_shufpd (__tmp, __tmp, _MM_SHUFFLE2 (0,1));
}

/* Store two DPFP values.  The address must be 16-byte aligned.  */
__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_store_pd (double *__P, __m128d __A)
{
#if __iset__ <= 4
   *(__m128d *) __builtin_assume_aligned (__P, 16) = __A;
#else /* __iset__ <= 4 */
  __builtin_storemas_128v ((__v2di) __A, __P, 0, __LCC_CHAN_ANY);
#endif /* __iset__ <= 4 */
}

/* Store two DPFP values.  The address need not be 16-byte aligned.  */
__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_storeu_pd (double *__P, __m128d __A)
{
  __builtin_ia32_storeupd (__P, __A);
}

/* Stores the lower DPFP value.  */
__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_store_sd (double *__P, __m128d __A)
{
  *__P = ((__v2df)__A)[0];
}

__E2K_INLINE double __DEFAULT_FUNCS_ATTRS
_mm_cvtsd_f64 (__m128d __A)
{
  return ((__v2df)__A)[0];
}

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_storel_pd (double *__P, __m128d __A)
{
  _mm_store_sd (__P, __A);
}

/* Stores the upper DPFP value.  */
__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_storeh_pd (double *__P, __m128d __A)
{
  *__P = ((__v2df)__A)[1];
}

/* Store the lower DPFP value across two words.
   The address must be 16-byte aligned.  */
__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_store1_pd (double *__P, __m128d __A)
{
  _mm_store_pd (__P, __builtin_ia32_shufpd (__A, __A, _MM_SHUFFLE2 (0,0)));
}

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_store_pd1 (double *__P, __m128d __A)
{
  _mm_store1_pd (__P, __A);
}

/* Store two DPFP values in reverse order.  The address must be aligned.  */
__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_storer_pd (double *__P, __m128d __A)
{
  _mm_store_pd (__P, __builtin_ia32_shufpd (__A, __A, _MM_SHUFFLE2 (0,1)));
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_cvtsi128_si32 (__m128i __A)
{
  return __builtin_ia32_vec_ext_v4si ((__v4si)__A, 0);
}

/* Intel intrinsic.  */
__E2K_INLINE long long __DEFAULT_FUNCS_ATTRS
_mm_cvtsi128_si64 (__m128i __A)
{
  return ((__v2di)__A)[0];
}

/* Microsoft intrinsic.  */
__E2K_INLINE long long __DEFAULT_FUNCS_ATTRS
_mm_cvtsi128_si64x (__m128i __A)
{
  return ((__v2di)__A)[0];
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_add_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_addpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_add_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_addsd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_sub_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_subpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_sub_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_subsd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_mul_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_mulpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_mul_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_mulsd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_div_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_divpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_div_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_divsd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_sqrt_pd (__m128d __A)
{
  return (__m128d)__builtin_ia32_sqrtpd ((__v2df)__A);
}

/* Return pair {sqrt (B[0), A[1]}.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_sqrt_sd (__m128d __A, __m128d __B)
{
  __v2df __tmp = __builtin_ia32_movsd ((__v2df)__A, (__v2df)__B);
  return (__m128d)__builtin_ia32_sqrtsd ((__v2df)__tmp);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_min_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_minpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_min_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_minsd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_max_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_maxpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_max_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_maxsd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_and_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_andpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_andnot_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_andnpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_or_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_orpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_xor_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_xorpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpeq_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpeqpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmplt_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpltpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmple_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmplepd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpgt_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpgtpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpge_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpgepd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpneq_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpneqpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpnlt_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpnltpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpnle_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpnlepd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpngt_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpngtpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpnge_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpngepd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpord_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpordpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpunord_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpunordpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpeq_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpeqsd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmplt_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpltsd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmple_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmplesd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpgt_sd (__m128d __A, __m128d __B)
{
  return (__m128d) __builtin_ia32_movsd ((__v2df) __A,
                                         (__v2df)
                                         __builtin_ia32_cmpltsd ((__v2df) __B,
                                                                 (__v2df) __A));
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpge_sd (__m128d __A, __m128d __B)
{
  return (__m128d) __builtin_ia32_movsd ((__v2df) __A,
                                         (__v2df)
                                         __builtin_ia32_cmplesd ((__v2df) __B,
                                                                 (__v2df) __A));
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpneq_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpneqsd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpnlt_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpnltsd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpnle_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpnlesd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpngt_sd (__m128d __A, __m128d __B)
{
  return (__m128d) __builtin_ia32_movsd ((__v2df) __A,
                                         (__v2df)
                                         __builtin_ia32_cmpnltsd ((__v2df) __B,
                                                                  (__v2df) __A));
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpnge_sd (__m128d __A, __m128d __B)
{
  return (__m128d) __builtin_ia32_movsd ((__v2df) __A,
                                         (__v2df)
                                         __builtin_ia32_cmpnlesd ((__v2df) __B,
                                                                  (__v2df) __A));
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpord_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpordsd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cmpunord_sd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_cmpunordsd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_comieq_sd (__m128d __A, __m128d __B)
{
  return __builtin_ia32_comisdeq ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_comilt_sd (__m128d __A, __m128d __B)
{
  return __builtin_ia32_comisdlt ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_comile_sd (__m128d __A, __m128d __B)
{
  return __builtin_ia32_comisdle ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_comigt_sd (__m128d __A, __m128d __B)
{
  return __builtin_ia32_comisdgt ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_comige_sd (__m128d __A, __m128d __B)
{
  return __builtin_ia32_comisdge ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_comineq_sd (__m128d __A, __m128d __B)
{
  return __builtin_ia32_comisdneq ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_ucomieq_sd (__m128d __A, __m128d __B)
{
  return __builtin_ia32_ucomisdeq ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_ucomilt_sd (__m128d __A, __m128d __B)
{
  return __builtin_ia32_ucomisdlt ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_ucomile_sd (__m128d __A, __m128d __B)
{
  return __builtin_ia32_ucomisdle ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_ucomigt_sd (__m128d __A, __m128d __B)
{
  return __builtin_ia32_ucomisdgt ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_ucomige_sd (__m128d __A, __m128d __B)
{
  return __builtin_ia32_ucomisdge ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_ucomineq_sd (__m128d __A, __m128d __B)
{
  return __builtin_ia32_ucomisdneq ((__v2df)__A, (__v2df)__B);
}

/* Create a vector of Qi, where i is the element number.  */

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_set_epi64x (long long __q1, long long __q0)
{
  return ((type_union_128) {l : { __q0, __q1 }}).__v2di;
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_set_epi64 (__m64 __q1,  __m64 __q0)
{
  return _mm_set_epi64x ((long long)__q1, (long long)__q0);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_set_epi32 (int __q3, int __q2, int __q1, int __q0)
{
  return (__m128i) ((type_union_128) {i : { __q0, __q1, __q2, __q3 }}).__v4si;
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_set_epi16 (short __q7, short __q6, short __q5, short __q4,
               short __q3, short __q2, short __q1, short __q0)
{
  return (__m128i) ((type_union_128) {h : { __q0, __q1, __q2, __q3, __q4, __q5, __q6, __q7 }}).__v8hi;
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_set_epi8 (char __q15, char __q14, char __q13, char __q12,
              char __q11, char __q10, char __q09, char __q08,
              char __q07, char __q06, char __q05, char __q04,
              char __q03, char __q02, char __q01, char __q00)
{
  return (__m128i) ((type_union_128) {c : { __q00, __q01, __q02, __q03, __q04, __q05, __q06, __q07,
                                            __q08, __q09, __q10, __q11, __q12, __q13, __q14, __q15 }}).__v16qi;
}

/* Set all of the elements of the vector to A.  */

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_set1_epi64x (long long __A)
{
  return _mm_set_epi64x (__A, __A);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_set1_epi64 (__m64 __A)
{
  return _mm_set_epi64 (__A, __A);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_set1_epi32 (int __A)
{
  return _mm_set_epi32 (__A, __A, __A, __A);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_set1_epi16 (short __A)
{
  type_union_128 dst;
  dst.l.l0 = dst.l.l1 = (__di) _mm_set1_pi16 (__A);
  return (__m128i) dst.__v8hi;
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_set1_epi8 (char __A)
{
  type_union_128 dst;
  dst.l.l0 = dst.l.l1 = (__di) _mm_set1_pi8 (__A);
  return (__m128i) dst.__v16qi;
}

/* Create a vector of Qi, where i is the element number.
   The parameter order is reversed from the _mm_set_epi* functions.  */

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_setr_epi64 (__m64 __q0, __m64 __q1)
{
  return _mm_set_epi64 (__q1, __q0);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_setr_epi32 (int __q0, int __q1, int __q2, int __q3)
{
  return _mm_set_epi32 (__q3, __q2, __q1, __q0);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_setr_epi16 (short __q0, short __q1, short __q2, short __q3,
                short __q4, short __q5, short __q6, short __q7)
{
  return _mm_set_epi16 (__q7, __q6, __q5, __q4, __q3, __q2, __q1, __q0);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_setr_epi8 (char __q00, char __q01, char __q02, char __q03,
               char __q04, char __q05, char __q06, char __q07,
               char __q08, char __q09, char __q10, char __q11,
               char __q12, char __q13, char __q14, char __q15)
{
  return _mm_set_epi8 (__q15, __q14, __q13, __q12, __q11, __q10, __q09, __q08,
                       __q07, __q06, __q05, __q04, __q03, __q02, __q01, __q00);
}

/* Create a vector with element 0 as *P and the rest zero.  */

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_load_si128 (__m128i const *__P)
{
#if __iset__ <= 4
  return *(__m128i *) __builtin_assume_aligned (__P, 16);
#else /* __iset__ <= 4 */
  return (__m128i) __builtin_loadmas_128v (__P, 0, __LCC_CHAN_ANY);
#endif /* __iset__ <= 4 */
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_loadu_si128 (__m128i_u const *__P)
{
  return (__m128i) __builtin_ia32_loaddqu ((char const *)__P);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_loadl_epi64 (__m128i_u const *__P)
{
#ifdef __ALIGNED__
  __di dst;
  dst  = (__di) ((unsigned char *) __P)[0];
  dst |= (__di) ((unsigned char *) __P)[1] << 8;
  dst |= (__di) ((unsigned char *) __P)[2] << 16;
  dst |= (__di) ((unsigned char *) __P)[3] << 24;
  dst |= (__di) ((unsigned char *) __P)[4] << 32;
  dst |= (__di) ((unsigned char *) __P)[5] << 40;
  dst |= (__di) ((unsigned char *) __P)[6] << 48;
  dst |= (__di) ((unsigned char *) __P)[7] << 56;
  return _mm_set_epi64 ((__m64)0LL, (__m64)dst);
#else /* __ALIGNED__ */
  return _mm_set_epi64 ((__m64)0LL, *(__m64_u *)__P);
#endif /* __ALIGNED__ */
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_loadu_si64 (void const *__P)
{
  return _mm_loadl_epi64 ((__m128i_u *)__P);
}

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_store_si128 (__m128i *__P, __m128i __B)
{
#if __iset__ <= 4
   *(__m128i *) __builtin_assume_aligned (__P, 16) = __B;
#else /* __iset__ <= 4 */
  __builtin_storemas_128v ((__v2di) __B, __P, 0, __LCC_CHAN_ANY);
#endif /* __iset__ <= 4 */
}

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_storeu_si128 (__m128i_u *__P, __m128i __B)
{
  __builtin_ia32_storedqu ((char *)__P, (__v16qi)__B);
}

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_storel_epi64 (__m128i_u *__P, __m128i __B)
{
#ifdef __ALIGNED__
  long long __dst = __builtin_ia32_vec_ext_v2di ((__v2di)__B, 0);
  ((unsigned char *)__P)[0] = __dst;
  ((unsigned char *)__P)[1] = __dst >> 8;
  ((unsigned char *)__P)[2] = __dst >> 16;
  ((unsigned char *)__P)[3] = __dst >> 24;
  ((unsigned char *)__P)[4] = __dst >> 32;
  ((unsigned char *)__P)[5] = __dst >> 40;
  ((unsigned char *)__P)[6] = __dst >> 48;
  ((unsigned char *)__P)[7] = __dst >> 56;
#else /* __ALIGNED__ */
  *(__m64_u *)__P = (__m64) ((__v2di)__B)[0];
#endif /* __ALIGNED__ */
}

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_storeu_si64 (void *__P, __m128i __B)
{
  _mm_storel_epi64 ((__m128i_u *)__P, __B);
}

__E2K_INLINE __m64 __DEFAULT_FUNCS_ATTRS
_mm_movepi64_pi64 (__m128i __B)
{
  return (__m64) ((__v2di)__B)[0];
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_movpi64_epi64 (__m64 __A)
{
  return _mm_set_epi64 ((__m64)0LL, __A);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_move_epi64 (__m128i __A)
{
  return (__m128i)__builtin_ia32_movq128 ((__v2di) __A);
}

/* Create an undefined vector.  */
__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_undefined_si128 (void)
{
  __m128i __Y = __Y;
  return __Y;
}

/* Create a vector of zeros.  */
__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_setzero_si128 (void)
{
  return (__m128i) ((type_union_128) {l : { 0, 0 }}).__v4si;
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cvtepi32_pd (__m128i __A)
{
  return (__m128d)__builtin_ia32_cvtdq2pd ((__v4si) __A);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_cvtepi32_ps (__m128i __A)
{
  return (__m128)__builtin_ia32_cvtdq2ps ((__v4si) __A);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cvtpd_epi32 (__m128d __A)
{
  return (__m128i)__builtin_ia32_cvtpd2dq ((__v2df) __A);
}

__E2K_INLINE __m64 __DEFAULT_FUNCS_ATTRS
_mm_cvtpd_pi32 (__m128d __A)
{
  return (__m64)__builtin_ia32_cvtpd2pi ((__v2df) __A);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_cvtpd_ps (__m128d __A)
{
  return (__m128)__builtin_ia32_cvtpd2ps ((__v2df) __A);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cvttpd_epi32 (__m128d __A)
{
  return (__m128i)__builtin_ia32_cvttpd2dq ((__v2df) __A);
}

__E2K_INLINE __m64 __DEFAULT_FUNCS_ATTRS
_mm_cvttpd_pi32 (__m128d __A)
{
  return (__m64)__builtin_ia32_cvttpd2pi ((__v2df) __A);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cvtpi32_pd (__m64 __A)
{
  return (__m128d)__builtin_ia32_cvtpi2pd ((__v2si) __A);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cvtps_epi32 (__m128 __A)
{
  return (__m128i)__builtin_ia32_cvtps2dq ((__v4sf) __A);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cvttps_epi32 (__m128 __A)
{
  return (__m128i)__builtin_ia32_cvttps2dq ((__v4sf) __A);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cvtps_pd (__m128 __A)
{
  return (__m128d)__builtin_ia32_cvtps2pd ((__v4sf) __A);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_cvtsd_si32 (__m128d __A)
{
  return __builtin_ia32_cvtsd2si ((__v2df) __A);
}

/* Intel intrinsic.  */
__E2K_INLINE long long __DEFAULT_FUNCS_ATTRS
_mm_cvtsd_si64 (__m128d __A)
{
  return __builtin_ia32_cvtsd2si64 ((__v2df) __A);
}

/* Microsoft intrinsic.  */
__E2K_INLINE long long __DEFAULT_FUNCS_ATTRS
_mm_cvtsd_si64x (__m128d __A)
{
  return __builtin_ia32_cvtsd2si64 ((__v2df) __A);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_cvttsd_si32 (__m128d __A)
{
  return __builtin_ia32_cvttsd2si ((__v2df) __A);
}

/* Intel intrinsic.  */
__E2K_INLINE long long __DEFAULT_FUNCS_ATTRS
_mm_cvttsd_si64 (__m128d __A)
{
  return __builtin_ia32_cvttsd2si64 ((__v2df) __A);
}

/* Microsoft intrinsic.  */
__E2K_INLINE long long __DEFAULT_FUNCS_ATTRS
_mm_cvttsd_si64x (__m128d __A)
{
  return __builtin_ia32_cvttsd2si64 ((__v2df) __A);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_cvtsd_ss (__m128 __A, __m128d __B)
{
  return (__m128)__builtin_ia32_cvtsd2ss ((__v4sf) __A, (__v2df) __B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cvtsi32_sd (__m128d __A, int __B)
{
  return (__m128d)__builtin_ia32_cvtsi2sd ((__v2df) __A, __B);
}

/* Intel intrinsic.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cvtsi64_sd (__m128d __A, long long __B)
{
  return (__m128d)__builtin_ia32_cvtsi642sd ((__v2df) __A, __B);
}

/* Microsoft intrinsic.  */
__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cvtsi64x_sd (__m128d __A, long long __B)
{
  return (__m128d)__builtin_ia32_cvtsi642sd ((__v2df) __A, __B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_cvtss_sd (__m128d __A, __m128 __B)
{
  return (__m128d)__builtin_ia32_cvtss2sd ((__v2df) __A, (__v4sf)__B);
}

#define _mm_shuffle_pd(A, B, N)                                         \
  ((__m128d)__builtin_ia32_shufpd ((__v2df)(__m128d)(A),                \
                                   (__v2df)(__m128d)(B), (int)(N)))

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_unpackhi_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_unpckhpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_unpacklo_pd (__m128d __A, __m128d __B)
{
  return (__m128d)__builtin_ia32_unpcklpd ((__v2df)__A, (__v2df)__B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_loadh_pd (__m128d __A, double const *__B)
{
  return (__m128d)__builtin_ia32_loadhpd ((__v2df)__A, __B);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_loadl_pd (__m128d __A, double const *__B)
{
  return (__m128d)__builtin_ia32_loadlpd ((__v2df)__A, __B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_movemask_pd (__m128d __A)
{
  return __builtin_ia32_movmskpd ((__v2df)__A);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_packs_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_packsswb128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_packs_epi32 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_packssdw128 ((__v4si)__A, (__v4si)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_packus_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_packuswb128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_unpackhi_epi8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_punpckhbw128 ((__v16qi)__A, (__v16qi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_unpackhi_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_punpckhwd128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_unpackhi_epi32 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_punpckhdq128 ((__v4si)__A, (__v4si)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_unpackhi_epi64 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_punpckhqdq128 ((__v2di)__A, (__v2di)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_unpacklo_epi8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_punpcklbw128 ((__v16qi)__A, (__v16qi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_unpacklo_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_punpcklwd128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_unpacklo_epi32 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_punpckldq128 ((__v4si)__A, (__v4si)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_unpacklo_epi64 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_punpcklqdq128 ((__v2di)__A, (__v2di)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_add_epi8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_paddb128 ((__v16qi)__A, (__v16qi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_add_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_paddw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_add_epi32 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_paddd128 ((__v4si)__A, (__v4si)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_add_epi64 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_paddq128 ((__v2di)__A, (__v2di)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_adds_epi8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_paddsb128 ((__v16qi)__A, (__v16qi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_adds_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_paddsw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_adds_epu8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_paddusb128 ((__v16qi)__A, (__v16qi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_adds_epu16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_paddusw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_sub_epi8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psubb128 ((__v16qi)__A, (__v16qi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_sub_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psubw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_sub_epi32 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psubd128 ((__v4si)__A, (__v4si)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_sub_epi64 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psubq128 ((__v2di)__A, (__v2di)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_subs_epi8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psubsb128 ((__v16qi)__A, (__v16qi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_subs_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psubsw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_subs_epu8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psubusb128 ((__v16qi)__A, (__v16qi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_subs_epu16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psubusw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_madd_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pmaddwd128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_mulhi_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pmulhw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_mullo_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pmullw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m64 __DEFAULT_FUNCS_ATTRS
_mm_mul_su32 (__m64 __A, __m64 __B)
{
  return (__m64)__builtin_ia32_pmuludq ((__v2si)__A, (__v2si)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_mul_epu32 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pmuludq128 ((__v4si)__A, (__v4si)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_slli_epi16 (__m128i __A, int __B)
{
  return (__m128i)__builtin_ia32_psllwi128 ((__v8hi)__A, __B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_slli_epi32 (__m128i __A, int __B)
{
  return (__m128i)__builtin_ia32_pslldi128 ((__v4si)__A, __B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_slli_epi64 (__m128i __A, int __B)
{
  return (__m128i)__builtin_ia32_psllqi128 ((__v2di)__A, __B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_srai_epi16 (__m128i __A, int __B)
{
  return (__m128i)__builtin_ia32_psrawi128 ((__v8hi)__A, __B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_srai_epi32 (__m128i __A, int __B)
{
  return (__m128i)__builtin_ia32_psradi128 ((__v4si)__A, __B);
}

#define _mm_bsrli_si128(A, N) \
  ((__m128i)__builtin_ia32_psrldqi128 ((__m128i)(A), (int)(N) * 8))
#define _mm_bslli_si128(A, N) \
  ((__m128i)__builtin_ia32_pslldqi128 ((__m128i)(A), (int)(N) * 8))
#define _mm_srli_si128(A, N) \
  ((__m128i)__builtin_ia32_psrldqi128 ((__m128i)(A), (int)(N) * 8))
#define _mm_slli_si128(A, N) \
  ((__m128i)__builtin_ia32_pslldqi128 ((__m128i)(A), (int)(N) * 8))

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_srli_epi16 (__m128i __A, int __B)
{
  return (__m128i)__builtin_ia32_psrlwi128 ((__v8hi)__A, __B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_srli_epi32 (__m128i __A, int __B)
{
  return (__m128i)__builtin_ia32_psrldi128 ((__v4si)__A, __B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_srli_epi64 (__m128i __A, int __B)
{
  return (__m128i)__builtin_ia32_psrlqi128 ((__v2di)__A, __B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_sll_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psllw128((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_sll_epi32 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pslld128((__v4si)__A, (__v4si)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_sll_epi64 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psllq128((__v2di)__A, (__v2di)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_sra_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psraw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_sra_epi32 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psrad128 ((__v4si)__A, (__v4si)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_srl_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psrlw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_srl_epi32 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psrld128 ((__v4si)__A, (__v4si)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_srl_epi64 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psrlq128 ((__v2di)__A, (__v2di)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_and_si128 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pand128 ((__v2di)__A, (__v2di)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_andnot_si128 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pandn128 ((__v2di)__A, (__v2di)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_or_si128 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_por128 ((__v2di)__A, (__v2di)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_xor_si128 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pxor128 ((__v2di)__A, (__v2di)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cmpeq_epi8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pcmpeqb128 ((__v16qi)__A, (__v16qi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cmpeq_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pcmpeqw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cmpeq_epi32 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pcmpeqd128 ((__v4si)__A, (__v4si)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cmplt_epi8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pcmpgtb128 ((__v16qs)__B, (__v16qs)__A);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cmplt_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pcmpgtw128 ((__v8hi)__B, (__v8hi)__A);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cmplt_epi32 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pcmpgtd128 ((__v4si)__B, (__v4si)__A);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cmpgt_epi8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pcmpgtb128 ((__v16qs)__A, (__v16qs)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cmpgt_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pcmpgtw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cmpgt_epi32 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pcmpgtd128 ((__v4si)__A, (__v4si)__B);
}

#define _mm_extract_epi16(A, N) \
  ((int) (unsigned short) __builtin_ia32_vec_ext_v8hi ((__v8hi)(__m128i)(A), (int)(N)))
#define _mm_insert_epi16(A, D, N)                               \
  ((__m128i) __builtin_ia32_vec_set_v8hi ((__v8hi)(__m128i)(A), \
                                          (int)(D), (int)(N)))

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_max_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pmaxsw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_max_epu8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pmaxub128 ((__v16qi)__A, (__v16qi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_min_epi16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pminsw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_min_epu8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pminub128 ((__v16qi)__A, (__v16qi)__B);
}

__E2K_INLINE int __DEFAULT_FUNCS_ATTRS
_mm_movemask_epi8 (__m128i __A)
{
  return __builtin_ia32_pmovmskb128 ((__v16qi)__A);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_mulhi_epu16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pmulhuw128 ((__v8hi)__A, (__v8hi)__B);
}

#define _mm_shufflehi_epi16(A, N) \
  ((__m128i)__builtin_ia32_pshufhw ((__v8hi)(__m128i)(A), (int)(N)))
#define _mm_shufflelo_epi16(A, N) \
  ((__m128i)__builtin_ia32_pshuflw ((__v8hi)(__m128i)(A), (int)(N)))
#define _mm_shuffle_epi32(A, N) \
  ((__m128i)__builtin_ia32_pshufd ((__v4si)(__m128i)(A), (int)(N)))

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_maskmoveu_si128 (__m128i __A, __m128i __B, char *__C)
{
  __builtin_ia32_maskmovdqu ((__v16qi)__A, (__v16qi)__B, __C);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_avg_epu8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pavgb128 ((__v16qi)__A, (__v16qi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_avg_epu16 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_pavgw128 ((__v8hi)__A, (__v8hi)__B);
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_sad_epu8 (__m128i __A, __m128i __B)
{
  return (__m128i)__builtin_ia32_psadbw128 ((__v16qi)__A, (__v16qi)__B);
}

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_stream_si32 (int *__A, int __B)
{
  __builtin_ia32_movnti (__A, __B);
}

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_stream_si64 (long long int *__A, long long int __B)
{
  __builtin_ia32_movnti64 (__A, __B);
}

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_stream_si128 (__m128i *__A, __m128i __B)
{
  __builtin_ia32_movntdq ((__v2di *)__A, (__v2di)__B);
}

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_stream_pd (double *__A, __m128d __B)
{
  __builtin_ia32_movntpd (__A, (__v2df)__B);
}

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_clflush (void const *__A)
{
  __builtin_ia32_clflush (__A);
}

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_lfence (void)
{
  __builtin_ia32_lfence ();
}

__E2K_INLINE void __DEFAULT_FUNCS_ATTRS
_mm_mfence (void)
{
  __builtin_ia32_mfence ();
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cvtsi32_si128 (int __A)
{
  return _mm_set_epi32 (0, 0, 0, __A);
}

/* Intel intrinsic.  */
__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cvtsi64_si128 (long long __A)
{
  return _mm_set_epi64x (0, __A);
}

/* Microsoft intrinsic.  */
__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_cvtsi64x_si128 (long long __A)
{
  return _mm_set_epi64x (0, __A);
}

/* Casts between various SP, DP, INT vector types.  Note that these do no
   conversion of values, they just change the type.  */
__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_castpd_ps(__m128d __A)
{
  return (__m128) __A;
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_castpd_si128(__m128d __A)
{
  return (__m128i) __A;
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_castps_pd(__m128 __A)
{
  return (__m128d) __A;
}

__E2K_INLINE __m128i __DEFAULT_FUNCS_ATTRS
_mm_castps_si128(__m128 __A)
{
  return (__m128i) __A;
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_castsi128_ps(__m128i __A)
{
  return (__m128) __A;
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_castsi128_pd(__m128i __A)
{
  return (__m128d) __A;
}

/* __DISABLE_SSE2__ */

#undef __DEFAULT_FUNCS_ATTRS

#endif /* _EMMINTRIN_H_INCLUDED */
