/* Copyright (c) 2018-2022 ZAO "MCST". All rights reserved. */

#ifndef _X86INTRIN_H_INCLUDED
# error "Never use <fma4intrin.h> directly; include <x86intrin.h> instead."
#endif

#ifndef _FMA4INTRIN_H_INCLUDED
#define _FMA4INTRIN_H_INCLUDED

/* We need definitions from the SSE4A, SSE3, SSE2 and SSE header files.  */
#include <ammintrin.h>

/* __FMA4__ */

/* Define the default attributes for the functions in this file */
#define __DEFAULT_FUNCS_ATTRS __attribute__((/*__gnu_inline__,*/ __always_inline__, __artificial__))

/* 128b Floating point multiply/add type instructions.  */
__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_macc_ps (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_ia32_vfmaddps ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_macc_pd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_ia32_vfmaddpd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_macc_ss (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_ia32_vfmaddss ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_macc_sd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_ia32_vfmaddsd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_msub_ps (__m128 __A, __m128 __B, __m128 __C)

{
  return (__m128) __builtin_e2k_vfmsubps ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_msub_pd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfmsubpd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_msub_ss (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_e2k_vfmsubss ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_msub_sd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfmsubsd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_nmacc_ps (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_e2k_vfnmaddps ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_nmacc_pd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfnmaddpd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_nmacc_ss (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_e2k_vfnmaddss ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_nmacc_sd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfnmaddsd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_nmsub_ps (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_e2k_vfnmsubps ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_nmsub_pd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfnmsubpd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_nmsub_ss (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_e2k_vfnmsubss ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_nmsub_sd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfnmsubsd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_maddsub_ps (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_ia32_vfmaddsubps ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_maddsub_pd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_ia32_vfmaddsubpd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_msubadd_ps (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_e2k_vfmsubaddps ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_msubadd_pd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfmsubaddpd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

/* 256b Floating point multiply/add type instructions.  */
__E2K_INLINE __m256 __DEFAULT_FUNCS_ATTRS
_mm256_macc_ps (__m256 __A, __m256 __B, __m256 __C)
{
  return (__m256) __builtin_ia32_vfmaddps256 ((__v8sf)__A, (__v8sf)__B, (__v8sf)__C);
}

__E2K_INLINE __m256d __DEFAULT_FUNCS_ATTRS
_mm256_macc_pd (__m256d __A, __m256d __B, __m256d __C)
{
  return (__m256d) __builtin_ia32_vfmaddpd256 ((__v4df)__A, (__v4df)__B, (__v4df)__C);
}

__E2K_INLINE __m256 __DEFAULT_FUNCS_ATTRS
_mm256_msub_ps (__m256 __A, __m256 __B, __m256 __C)
{
  return (__m256) __builtin_e2k_vfmsubps256 ((__v8sf)__A, (__v8sf)__B, (__v8sf)__C);
}

__E2K_INLINE __m256d __DEFAULT_FUNCS_ATTRS
_mm256_msub_pd (__m256d __A, __m256d __B, __m256d __C)
{
  return (__m256d) __builtin_e2k_vfmsubpd256 ((__v4df)__A, (__v4df)__B, (__v4df)__C);
}

__E2K_INLINE __m256 __DEFAULT_FUNCS_ATTRS
_mm256_nmacc_ps (__m256 __A, __m256 __B, __m256 __C)
{
  return (__m256) __builtin_e2k_vfnmaddps256 ((__v8sf)__A, (__v8sf)__B, (__v8sf)__C);
}

__E2K_INLINE __m256d __DEFAULT_FUNCS_ATTRS
_mm256_nmacc_pd (__m256d __A, __m256d __B, __m256d __C)
{
  return (__m256d) __builtin_e2k_vfnmaddpd256 ((__v4df)__A, (__v4df)__B, (__v4df)__C);
}

__E2K_INLINE __m256 __DEFAULT_FUNCS_ATTRS
_mm256_nmsub_ps (__m256 __A, __m256 __B, __m256 __C)
{
  return (__m256) __builtin_e2k_vfnmsubps256 ((__v8sf)__A, (__v8sf)__B, (__v8sf)__C);
}

__E2K_INLINE __m256d __DEFAULT_FUNCS_ATTRS
_mm256_nmsub_pd (__m256d __A, __m256d __B, __m256d __C)
{
  return (__m256d) __builtin_e2k_vfnmsubpd256 ((__v4df)__A, (__v4df)__B, (__v4df)__C);
}

__E2K_INLINE __m256 __DEFAULT_FUNCS_ATTRS
_mm256_maddsub_ps (__m256 __A, __m256 __B, __m256 __C)
{
  return (__m256) __builtin_ia32_vfmaddsubps256 ((__v8sf)__A, (__v8sf)__B, (__v8sf)__C);
}

__E2K_INLINE __m256d __DEFAULT_FUNCS_ATTRS
_mm256_maddsub_pd (__m256d __A, __m256d __B, __m256d __C)
{
  return (__m256d) __builtin_ia32_vfmaddsubpd256 ((__v4df)__A, (__v4df)__B, (__v4df)__C);
}

__E2K_INLINE __m256 __DEFAULT_FUNCS_ATTRS
_mm256_msubadd_ps (__m256 __A, __m256 __B, __m256 __C)
{
  return (__m256) __builtin_e2k_vfmsubaddps256 ((__v8sf)__A, (__v8sf)__B, (__v8sf)__C);
}

__E2K_INLINE __m256d __DEFAULT_FUNCS_ATTRS
_mm256_msubadd_pd (__m256d __A, __m256d __B, __m256d __C)
{
  return (__m256d) __builtin_e2k_vfmsubaddpd256 ((__v4df)__A, (__v4df)__B, (__v4df)__C);
}

/* __DISABLE_FMA4__ */

#undef __DEFAULT_FUNCS_ATTRS

#endif /* _FMA4INTRIN_H_INCLUDED */
