/* Copyright (c) 2018-2022 ZAO "MCST". All rights reserved. */

#ifndef _IMMINTRIN_H_INCLUDED
# error "Never use <fmaintrin.h> directly; include <immintrin.h> instead."
#endif

#ifndef _FMAINTRIN_H_INCLUDED
#define _FMAINTRIN_H_INCLUDED

/* __FMA__ */

/* Define the default attributes for the functions in this file */
#define __DEFAULT_FUNCS_ATTRS __attribute__((/*__gnu_inline__,*/ __always_inline__, __artificial__))

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_fmadd_pd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d)__builtin_ia32_vfmaddpd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m256d __DEFAULT_FUNCS_ATTRS
_mm256_fmadd_pd (__m256d __A, __m256d __B, __m256d __C)
{
  return (__m256d)__builtin_ia32_vfmaddpd256 ((__v4df)__A, (__v4df)__B, (__v4df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_fmadd_ps (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128)__builtin_ia32_vfmaddps ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m256 __DEFAULT_FUNCS_ATTRS
_mm256_fmadd_ps (__m256 __A, __m256 __B, __m256 __C)
{
  return (__m256)__builtin_ia32_vfmaddps256 ((__v8sf)__A, (__v8sf)__B, (__v8sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_fmadd_sd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_ia32_vfmaddsd3 ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_fmadd_ss (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_ia32_vfmaddss3 ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_fmsub_pd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfmsubpd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m256d __DEFAULT_FUNCS_ATTRS
_mm256_fmsub_pd (__m256d __A, __m256d __B, __m256d __C)
{
  return (__m256d) __builtin_e2k_vfmsubpd256 ((__v4df)__A, (__v4df)__B, (__v4df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_fmsub_ps (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_e2k_vfmsubps ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m256 __DEFAULT_FUNCS_ATTRS
_mm256_fmsub_ps (__m256 __A, __m256 __B, __m256 __C)
{
  return (__m256) __builtin_e2k_vfmsubps256 ((__v8sf)__A, (__v8sf)__B, (__v8sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_fmsub_sd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfmsubsd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_fmsub_ss (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_e2k_vfmsubss ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_fnmadd_pd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfnmaddpd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m256d __DEFAULT_FUNCS_ATTRS
_mm256_fnmadd_pd (__m256d __A, __m256d __B, __m256d __C)
{
  return (__m256d) __builtin_e2k_vfnmaddpd256 ((__v4df)__A, (__v4df)__B, (__v4df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_fnmadd_ps (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_e2k_vfnmaddps ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m256 __DEFAULT_FUNCS_ATTRS
_mm256_fnmadd_ps (__m256 __A, __m256 __B, __m256 __C)
{
  return (__m256) __builtin_e2k_vfnmaddps256 ((__v8sf)__A, (__v8sf)__B, (__v8sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_fnmadd_sd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfnmaddsd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_fnmadd_ss (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_e2k_vfnmaddss ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_fnmsub_pd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfnmsubpd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m256d __DEFAULT_FUNCS_ATTRS
_mm256_fnmsub_pd (__m256d __A, __m256d __B, __m256d __C)
{
  return (__m256d) __builtin_e2k_vfnmsubpd256 ((__v4df)__A, (__v4df)__B, (__v4df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_fnmsub_ps (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_e2k_vfnmsubps ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m256 __DEFAULT_FUNCS_ATTRS
_mm256_fnmsub_ps (__m256 __A, __m256 __B, __m256 __C)
{
  return (__m256) __builtin_e2k_vfnmsubps256 ((__v8sf)__A, (__v8sf)__B, (__v8sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_fnmsub_sd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfnmsubsd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_fnmsub_ss (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_e2k_vfnmsubss ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_fmaddsub_pd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d)__builtin_ia32_vfmaddsubpd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m256d __DEFAULT_FUNCS_ATTRS
_mm256_fmaddsub_pd (__m256d __A, __m256d __B, __m256d __C)
{
  return (__m256d)__builtin_ia32_vfmaddsubpd256 ((__v4df)__A, (__v4df)__B, (__v4df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_fmaddsub_ps (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128)__builtin_ia32_vfmaddsubps ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m256 __DEFAULT_FUNCS_ATTRS
_mm256_fmaddsub_ps (__m256 __A, __m256 __B, __m256 __C)
{
  return (__m256)__builtin_ia32_vfmaddsubps256 ((__v8sf)__A, (__v8sf)__B, (__v8sf)__C);
}

__E2K_INLINE __m128d __DEFAULT_FUNCS_ATTRS
_mm_fmsubadd_pd (__m128d __A, __m128d __B, __m128d __C)
{
  return (__m128d) __builtin_e2k_vfmsubaddpd ((__v2df)__A, (__v2df)__B, (__v2df)__C);
}

__E2K_INLINE __m256d __DEFAULT_FUNCS_ATTRS
_mm256_fmsubadd_pd (__m256d __A, __m256d __B, __m256d __C)
{
  return (__m256d) __builtin_e2k_vfmsubaddpd256 ((__v4df)__A, (__v4df)__B, (__v4df)__C);
}

__E2K_INLINE __m128 __DEFAULT_FUNCS_ATTRS
_mm_fmsubadd_ps (__m128 __A, __m128 __B, __m128 __C)
{
  return (__m128) __builtin_e2k_vfmsubaddps ((__v4sf)__A, (__v4sf)__B, (__v4sf)__C);
}

__E2K_INLINE __m256 __DEFAULT_FUNCS_ATTRS
_mm256_fmsubadd_ps (__m256 __A, __m256 __B, __m256 __C)
{
  return (__m256) __builtin_e2k_vfmsubaddps256 ((__v8sf)__A, (__v8sf)__B, (__v8sf)__C);
}

/* __DISABLE_FMA__ */

#undef __DEFAULT_FUNCS_ATTRS

#endif /* _FMAINTRIN_H_INCLUDED */
